/* 
 * Copyright (c) 2009, 2010, Oracle and/or its affiliates. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of the
 * License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */

#include "geometry.h"

using namespace MySQL::Geometry;

//----------------- Rect ---------------------------------------------------------------------------

Rect::Rect()
{
  left= 0;
  top= 0;
  width= 0;
  height= 0;
}

//--------------------------------------------------------------------------------------------------

Rect::Rect(double x, double y, double w, double h)
{
  left= x;
  top= y;
  width= w;
  height= h;
}

//--------------------------------------------------------------------------------------------------

bool Rect::contains(double x, double y)
{
  return (x >= left) && (x <= left + width) &&
    (y >= top) && (y <= top + height);
}

//--------------------------------------------------------------------------------------------------

bool Rect::contains_flipped(double x, double y)
{
  // For cairo text the top is actually the bottom (when using it for hit tests).
  return (x >= left) && (x <= left + width) &&
    (y >= top - height) && (y <= top);
}

//--------------------------------------------------------------------------------------------------

/**
 * Inflates the rectangle by adding the given amounts to the left/top coordinate and subtracting them
 * from the right/bottom coordinate.
 */
void Rect::inflate(double horizontal, double vertical)
{
  left += horizontal;
  width -= 2 * horizontal;
  top += vertical;
  height -= 2 * vertical;
}

//--------------------------------------------------------------------------------------------------

double Rect::right()
{
  return left + width;
}

//--------------------------------------------------------------------------------------------------

double Rect::bottom()
{
  return top + height;
}

//----------------- Point --------------------------------------------------------------------------

Point::Point()
{
  left= 0;
  top= 0;
}

//--------------------------------------------------------------------------------------------------

Point::Point(double x, double y)
{
  left= x;
  top= y;
}

//--------------------------------------------------------------------------------------------------

