/* 
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of the
 * License.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */


#import "MFPopup.h"
#include "mforms/mforms.h"
#include <cairo/cairo-quartz.h>

@implementation MFPopupImpl

- (id)initWithObject:(::mforms::Popup*)popup
{
  self= [super initWithFrame:NSMakeRect(10,10,10,10)];
  if (self)
  {
    mOwner= popup;
    mOwner->set_data(self);
    mTag= 0;
  }
  return self;
}

- (BOOL)isFlipped
{
  return YES;
}

- (void)setTag:(NSInteger)tag
{
  mTag= tag;
}


- (NSInteger)tag
{
  return mTag;
}

- (void)drawRect:(NSRect)rect
{
  NSRect frame = [self frame];

  CGContextRef cgref = (CGContextRef)[[NSGraphicsContext currentContext] graphicsPort];

  cairo_surface_t *surface= cairo_quartz_surface_create_for_cg_context(cgref, NSWidth(frame), NSHeight(frame));
  
  cairo_t *cr = cairo_create(surface);
  try
  {
//    mOwner->repaint(cr, rect.origin.x, rect.origin.y, rect.size.width, rect.size.height);
  }
  catch (...)
  {
    cairo_destroy(cr);
    cairo_surface_destroy(surface);
    throw;
  }
  cairo_destroy(cr);
  cairo_surface_destroy(surface);
}

//--------------------------------------------------------------------------------------------------

/**
 * Helper to avoid code duplication.
 */
- (void) doMouseUp: (NSEvent*) event
{
  NSPoint p = [self convertPointFromBase: [event locationInWindow]];
  NSInteger button= [event buttonNumber];
  switch ([event clickCount])
  {
    case 1:
      mOwner->mouse_click(button, p.x, p.y);
      break;
    case 2:
      mOwner->mouse_double_click(button, p.x, p.y);
      break;
    default:
      mOwner->mouse_up(button, p.x, p.y);
      break;
  }
}

//--------------------------------------------------------------------------------------------------

/**
 * Helper to avoid code duplication.
 */
- (void) doMouseDown: (NSEvent*) event
{
  NSPoint p = [self convertPointFromBase: [event locationInWindow]];
  NSInteger button= [event buttonNumber];
  mOwner->mouse_down(button, p.x, p.y);
}

//--------------------------------------------------------------------------------------------------

- (void)mouseDown:(NSEvent*)event
{
  [self doMouseDown: event];
}

//--------------------------------------------------------------------------------------------------

- (void)mouseUp:(NSEvent*)event
{
  [self doMouseUp: event];
}

//--------------------------------------------------------------------------------------------------

- (void)otherMouseDown:(NSEvent*)event
{
  [self doMouseDown: event];
}

//--------------------------------------------------------------------------------------------------

- (void)otherMouseUp:(NSEvent*)event
{
  [self doMouseUp: event];
}

//--------------------------------------------------------------------------------------------------

- (void)rightMouseDown:(NSEvent*)event
{
  [self doMouseDown: event];
}

//--------------------------------------------------------------------------------------------------

- (void)rightMouseUp:(NSEvent*)event
{
  [self doMouseUp: event];
}

//--------------------------------------------------------------------------------------------------

- (void)mouseMoved:(NSEvent *)event
{
  NSPoint p = [self convertPointFromBase: [event locationInWindow]];
  mOwner->mouse_move(p.x, p.y);
}

//--------------------------------------------------------------------------------------------------

- (void)mouseEntered:(NSEvent *)event
{
  mOwner->mouse_enter();
}

//--------------------------------------------------------------------------------------------------

- (void)mouseExited:(NSEvent *)event
{
  mOwner->mouse_leave();
}

//--------------------------------------------------------------------------------------------------

- (BOOL)acceptsFirstResponder
{
  return YES;
}

//--------------------------------------------------------------------------------------------------

- (void) updateTrackingAreas
{
  [super updateTrackingAreas];

  // Create one tracking area which covers the whole control and make it get mouse events
  // only when the mouse is within the area but regardless of the first responder status.
  [self removeTrackingArea: mTrackingArea];
  [mTrackingArea release];
  
  NSTrackingAreaOptions options= NSTrackingMouseEnteredAndExited | NSTrackingMouseMoved | 
                                 NSTrackingCursorUpdate | NSTrackingActiveInActiveApp |
                                 NSTrackingInVisibleRect;
  mTrackingArea= [[NSTrackingArea alloc] initWithRect: [self bounds] options: options owner: self userInfo: nil];
  [self addTrackingArea: mTrackingArea];
}

//--------------------------------------------------------------------------------------------------

static bool popup_create(::mforms::Popup *self, ::mforms::PopupStyle style)
{
  [[[MFPopupImpl alloc] initWithObject:self] autorelease];
  
  return true;  
}

static void popup_set_needs_repaint(::mforms::Popup *self)
{
  [self->get_data() performSelectorOnMainThread: @selector(invalidate)
                                    withObject: nil
                                  waitUntilDone: NO];
}

static void popup_set_size(::mforms::Popup *self, int w, int h)
{
  
}

static int popup_show(::mforms::Popup *self, int x, int y)
{
  return 0;
}

static MySQL::Geometry::Rect popup_get_content_rect(::mforms::Popup *self)
{
  return MySQL::Geometry::Rect();
}


void cf_popup_init()
{
  ::mforms::ControlFactory *f = ::mforms::ControlFactory::get_instance();
  
  f->_popup_impl.create= &popup_create;
  f->_popup_impl.set_needs_repaint= &popup_set_needs_repaint;
  f->_popup_impl.set_size= &popup_set_size;
  f->_popup_impl.show= &popup_show;
  f->_popup_impl.get_content_rect= &popup_get_content_rect;
}

@end


