/*
 * Copyright (c) 2010, Oracle and/or its affiliates. All rights reserved.
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; version 2 of the
 * License.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA
 * 02110-1301  USA
 */
 
#include "../lf_popup.h"

namespace mforms {
namespace gtk {

PopupImpl::PopupImpl(::mforms::Popup *self)
  : ObjectImpl(self), _fixed_width(-1), _fixed_height(-1)
{
  _darea = Gtk::manage(new Gtk::EventBox());
  _darea->signal_expose_event().connect(sigc::bind(sigc::mem_fun(this, &PopupImpl::repaint), self));
  _darea->signal_button_press_event().connect_notify(sigc::bind(sigc::mem_fun(this, &PopupImpl::mouse_button_event), self));
  _darea->signal_button_release_event().connect_notify(sigc::bind(sigc::mem_fun(this, &PopupImpl::mouse_button_event), self));
  _darea->signal_enter_notify_event().connect_notify(sigc::bind(sigc::mem_fun(this, &PopupImpl::mouse_cross_event), self));
  _darea->signal_leave_notify_event().connect_notify(sigc::bind(sigc::mem_fun(this, &PopupImpl::mouse_cross_event), self));
  _darea->signal_motion_notify_event().connect_notify(sigc::bind(sigc::mem_fun(this, &PopupImpl::mouse_move_event), self));

  // request mouse moved events
  _darea->add_events(Gdk::POINTER_MOTION_MASK);
  
 // setup();
  
 // self->_repaint.connect(sigc::mem_fun(_darea, &Gtk::Widget::queue_draw));
}
  
  
void PopupImpl::set_size(int width, int height)
{
  _darea->set_size_request(width, height);
 // ViewImpl::set_size(width, height);
  
  _fixed_width = width;
  _fixed_height = height;
}


bool PopupImpl::repaint(GdkEventExpose *event, ::mforms::Popup *self)
{/*
  //This vv needs improvment on linux. Maybe setup an event listener which is bound to resize
  int w  = -1;
  int h = -1;
  self->get_layout_size(&w, &h);
  if (_fixed_height >= 0)
    h = _fixed_height;
  if (_fixed_width >= 0)
    w = _fixed_width;
  _darea->set_size_request(w, h); // Temp hack to allow drawing of box

  Cairo::RefPtr<Cairo::Context> context(_darea->get_window()->create_cairo_context());
  self->repaint(context->cobj(), event->area.x, event->area.y, event->area.width, event->area.height);
*/
  return true;
}

void PopupImpl::mouse_cross_event(GdkEventCrossing *event, ::mforms::Popup *self)
{
  if (event->type == GDK_ENTER_NOTIFY)
    self->mouse_enter();
  else
    self->mouse_leave();
}

void PopupImpl::mouse_button_event(GdkEventButton *event, ::mforms::Popup *self)
{
  if (event->type == GDK_BUTTON_PRESS)
  {
    self->mouse_down(event->button-1, (int)event->x, (int)event->y);
    self->mouse_click(event->button-1, (int)event->x, (int)event->y);
  }
  else if (event->type == GDK_BUTTON_RELEASE)
    self->mouse_up(event->button-1, (int)event->x, (int)event->y);
  else if (event->type == GDK_2BUTTON_PRESS)
    self->mouse_double_click(event->button-1, (int)event->x, (int)event->y);
}
  
void PopupImpl::mouse_move_event(GdkEventMotion *event, ::mforms::Popup *self)
{
  self->mouse_move((int)event->x, (int)event->y);
}

//bool PopupImpl::create(::mforms::Popup *self, ::mforms::PopupStyle style)
//{
//  return new PopupImpl(self) != 0;
//}

void PopupImpl::set_needs_repaint(::mforms::Popup *self)
{
  // request a repaint so that this can be called from any thread
  PopupImpl *impl = self->get_data<PopupImpl>();
 // impl->_darea->queue_redraw();
}

void PopupImpl::set_size(::mforms::Popup *, int, int)
{
  
}

int PopupImpl::show(::mforms::Popup *, int, int)
{
  return 0;
}

MySQL::Geometry::Rect PopupImpl::get_content_rect(::mforms::Popup *)
{
  return MySQL::Geometry::Rect();  
}

void PopupImpl::init()
{
/*
  ::mforms::gtk::ControlFactory *f = ::mforms::ControlFactory::get_instance();

  f->_popup_impl.create            = &PopupImpl::create;
  f->_popup_impl.set_needs_repaint = &PopupImpl::set_needs_repaint;
  f->_popup_impl.set_size          = &PopupImpl::set_size;
  f->_popup_impl.show              = &PopupImpl::show;
  f->_popup_impl.get_content_rect  = &PopupImpl::get_content_rect;
*/
}

};
};
